" Vim syntax file
" Language:	definable.cfg file
" Maintainer:	Pawe Paucha <pawel@praterm.com.pl>
" Last change:	2001 Nov 6
"
" $Id: definable.vim 790 2003-06-23 15:06:47Z pawel $
"
" SZARP 2.1
" Podwietlanie skadni dla pliku definable.cfg.
"

" Ten plik lepiej czyta si od dou ;-)

" Sprawdzana jest poprawno wyraenia w ONP (odwrotnej notacji polskiej). 
" Pomys polega na tym, eby grupowa kolejne pary liczba - operator
" dwuargumentowy, przy dopuszczalnych zagniedeniach. Na przyklad w 
" wyraeniu '5 4 3 * + 7 -' 5 zostawiamy na boku,  po czym grupujemy 4 z '+',
" 3 z '*' (zagniedenie) a 7 z '-'. Jeeli uda nam si w ten sposb wszystko
" zgrupowa, to wyraenie jest poprawne (na stosie zostaje jedna liczba).
" Szczeglnego traktowania wymagaj nastpujce operatory:
" '&' - zamiana wartoci na stosie - ignorowany
" '!' - dublowanie wartoci na stosie - jest traktowany jak liczba
"
" Rzeczy nie obsuone lub niesprawdzane :
" - nie sprawdzamy czy jest wystarczajca ilo argumentw dla operatora '&'
" - nie obsuguj operatorw warunku '?f' oraz wywoanie funkcji '$' - w tym
" drugim przypadku nie znamy iloci argumentw funkcji, w pierwszym przypadku
" wynika to z pewnych ogranicze Vim'a. Jeeli w formule wystpi jeden z 
" tych operatorw (podwietlany na czerwono), to formua jest kolorowana,
" ale NIE JEST SPRAWDZANA jej poprawno.

" Usuwamy ewentualne mieci
syn clear

" Ten operator moe wystpi samodzielnie - nie zmienia wielkoci stosu.
syn match rpn_ignore_op "[ ]\+&" contained

" Cz wewntrzna formuy - czymy liczb z pierwszym wystpujcym po niej
" operatorem, pozwalajc na zagniedanie. Operator '!' jest traktowany jako
" liczba.
syn region rpn_inner contained matchgroup=rpn_ind start="[ ]\+[0-9]\+" matchgroup=rpn_const start="[ ]\+#[-]\=[0-9]\+" matchgroup=operator start="[ ]\+!" end="[ ]\+[\+\-\/\*<>~^:N]" contains=rpn_inner,rpn_ignore_op

" A tu zaczynamy rozpoznawa formue w ONP - na pocztku musi by staa lub
" indeks w IPC, na kocu koniec wiersza, a w rodku cz wewntrzna formuy
syn region rpn matchgroup=rpn_ind start="=[0-9]\+"lc=1 matchgroup=rpn_const start="=#[-]\=[0-9]\+"lc=1 end="[ ]*$" contains=rpn_inner 

" To si w kilku miejscach przydaje - liczba na to.
syn match rpn_count "[0-9]\+" contained

" Tutaj obsugujemy formuy zawierajce warunki lub wywoania funkcji - 
" nie potrafimy sprawdzi ich poprawnoci, wic kolorujemy tylko odpowiednie
" elementy: operatory, spacje, indeksy w IPC, stae i operatory specjalne
" ($ i ?f).
syn match rpn_ops "[\+\-\/\*!&<>~^:N]" contained
syn match rpn_ind "[0-9]\+" contained
syn match rpn_space "[ ]\+" contained
syn match rpn_const "#[-]\=[0-9]\+" contained
syn match rpn_spec_op "[ ]\+\(\$\)\|\(?f\)" contained
syn match rpn_special "\([^\$]\+\$.*$\)\|\([^?]\+?f.*$\)" contains=rpn_spec_op,rpn_const,rpn_ind,rpn_space,rpn_ops contained

" Kolejne (od tyu) fragmenty linii z informacjami ptt.
syn match ptt_comm "#.*$"
syn match ptt_desc "[^#]\+" nextgroup=ptt_comm
syn match ptt_unit contained "\[[^\]]\+\]"
syn match ptt_colon contained ":"
syn match ptt_full "[^;]\+;" nextgroup=ptt_desc contains=ptt_unit,ptt_colon
syn match ptt_short "[^ ]\+[^ ]\+" nextgroup=ptt_full
syn match ptt_prec "[0-9]\+[ ]\+" nextgroup=ptt_short

" Jeli jest jaka inna linia, to jest to bd!
syn match error ".\+"

" Zaczynamy dopasowywanie informacji o ptt
syn match def_ptt "^def_par_ptt([0-9]\+)=" nextgroup=ptt_prec contains=rpn_count

" Zaczynamy dopasowywanie formuy
syn match rpn_start "^def_par_formula([0-9]\+)=" nextgroup=rpn,rpn_special contains=rpn_count

" Linie z komentarzami
syn match comment "^#.*$" 
" Pierwsze dwie linie
syn match def "^def_\(\(offset\)\|\(count\)\)=[0-9]\+$" contains=rpn_count

" Kolorki
if !exists("did_definable_syntax_inits")
  let did_definable_syntax_inits = 1
  hi link rpn		Error
  hi link rpn_equal	Keyword
  hi link rpn_outer	PreProc
  hi link rpn_inner	Error
  hi link rpn_start	Special
  hi link rpn_count	Keyword
  hi link rpn_ignore_op	Operator
  hi link rpn_special	Error
  hi link rpn_spec_op	Special
  hi link rpn_ops	Operator
  hi link ptt_comm	Comment
  hi link ptt_desc 	rpn_ind
  hi link ptt_full	Constant
  hi link ptt_colon	Special
  hi link ptt_unit	Keyword
  hi link ptt_short	rpn_const
  hi link ptt_prec	PreProc
  hi link def_ptt	Special
  hi link def		Special
  hi rpn_const	ctermfg=DarkGreen guifg=DarkGreen
  hi rpn_ind	ctermfg=Green guifg=Green
endif

let b:current_syntax = "definable"

" vim: ts=8
